# Julia editors

*Tutorial on collaborating on Julia packages: https://www.matecdev.com/posts/julia-package-collaboration.html*

## Emacs

There are a couple of addons that make working with Julia much nicer in Emacs:

1. `julia-mode` gives syntax highlighting. Install with `M-x package-install julia-mode`.

2. [`julia-snail`](https://github.com/gcv/julia-snail) provides IDE-like features, 
   especially a fully-functional REPL and the ability to evaluate code straight from inside a buffer.
   Note that the installation can be somewhat tricky. You first need to manually install
   all the dependencies of its dependency [vterm](https://github.com/akermu/emacs-libvterm),
   then install vterm itself with `M-x package-install vterm`, *before* you can do
   `M-x package-install julia-snail`. Then add it to your `init.el` with
   `(require 'julia-snail)` and `(add-hook 'julia-mode-hook #'julia-snail-mode)`.
   
3. [`company-mode`](http://company-mode.github.io/) integrates with Snail to give code 
   completion. Install with `M-x package-install company`, then add 
   `(add-hook 'julia-mode-hook #'company-mode)` and 
   `(global-set-key (kbd "C-<tab>") 'company-complete)` to your `init.el`.
   
4. [`magit`](https://magit.vc/) is a great git interface for Emacs. Install with
   `M-x package-install magit` and add `(global-set-key (kbd "C-x g") 'magit-status)`
   to your `init.el`.

## VSCode

See [here](https://www.julia-vscode.org/).

*TODO: add more detail.*


## Other useful stuff

1. [`Revise.jl`](https://timholy.github.io/Revise.jl/stable/) allows one to reload code
   without restarting the Julia interpreter. Get it with `Pkg.add("Revise")`, then 
   add `using Revise` to `.julia/config/startup.jl` to have it automatically available.

2. [`git-bug`](https://github.com/MichaelMure/git-bug) is a commandline bug tracker that
   integrates directly with git. It can also be used as a local interface to Gitlab issues.
   To install on Linux, download the [binary](https://github.com/MichaelMure/git-bug/releases/tag/v0.8.0),
   copy it to `/usr/local/bin/git-bug`, and set it as executable with `chmod +x 
   /usr/local/bin/git-bug`. To set up Gitlab integration, run `git bug bridge configure`
   from within the relevant repository.
   
