### Persephone - a socio-economic-ecological model of European agricultural landscapes.
###
### These are the tests for the core landscape functions.
###

@testset "Landscape initialisation" begin
    # initialise the landscape part of the model
    landscape = Persephone.initlandscape()
    space = GridSpace(size(landscape), periodic=false)
    properties = Dict{Symbol,Any}(:landscape=>landscape)
    model = AgentBasedModel(FarmPlot, space, properties=properties,
                            rng=Random.Xoshiro(param("core.seed")), warn=false)
    Persephone.initfields!(model)
    # these tests are specific to the Jena maps
    @test size(model.landscape) == (1754, 1602)
    @test Persephone.landcover(model, (100,100)) == Persephone.forest
    @test Persephone.landcover(model, (300,1)) == Persephone.soil
    @test Persephone.landcover(model, (500,1)) == Persephone.nodata
    @test Persephone.landcover(model, (400,400)) == Persephone.grass
    @test Persephone.landcover(model, (800,800)) == Persephone.agriculture
    @test Persephone.landcover(model, (1100,1100)) == Persephone.builtup
    @test Persephone.countfields(model) == 2092
    @test Persephone.averagefieldsize(model) == 5.37
    @test count(f -> ismissing(f.fieldid), model.landscape) == 1685573
    @test length(Persephone.farmplot(model, (800,800)).pixels) == 4049                
end

@testset "Event system" begin
    # initialise a basic model landscape
    landscape = Persephone.initlandscape()
    space = GridSpace(size(landscape), periodic=false)
    properties = Dict{Symbol,Any}(:date=>param("core.startdate"),
                                  :landscape=>landscape,
                                  :events=>Vector{FarmEvent}())
    model = AgentBasedModel(Union{Farmer,Animal,FarmPlot}, space, properties=properties,
                            rng=Random.Xoshiro(param("core.seed")), warn=false)
    # create some events and see whether they show up on the map and disappear as they should
    createevent!(model, [(1,1), (1,2), (1,3), (2,1), (2,3)], Persephone.tillage)
    createevent!(model, [(1,1), (1,2), (1,3), (2,2)], Persephone.sowing, 2)
    @test model.landscape[1,1].events == [Persephone.tillage, Persephone.sowing]
    @test model.landscape[2,1].events == [Persephone.tillage]
    @test model.landscape[2,2].events == [Persephone.sowing]
    stepsimulation!(model)
    @test model.landscape[1,1].events == [Persephone.sowing]
    @test model.landscape[2,1].events == []
    @test model.landscape[2,2].events == [Persephone.sowing]
    stepsimulation!(model)
    @test model.landscape[1,1].events == []
    @test model.landscape[2,1].events == []
    @test model.landscape[2,2].events == []
end
